@@ -28,6 +28,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <string.h>
 
+#include <gnuastro/ds9.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/table.h>
@@ -180,16 +181,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->filename)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->filename=arg;
+        if(arg[0]!='\0') p->filename=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -271,11 +273,15 @@ ui_read_check_only_options(struct tableparams *p)
               "v1x,v1y:v2x,v2y:v3x,v3y:...");
     }
 
-
-  /* Make sure '--head' and '--tail' aren't given together. */
-  if(p->head!=GAL_BLANK_SIZE_T && p->tail!=GAL_BLANK_SIZE_T)
-    error(EXIT_FAILURE, 0, "'--head' and '--tail' options cannot be "
-          "called together");
+  /* Make sure only one of the positional row selection operations is
+     called in this run.*/
+  if(p->rowlimit
+     && p->rowrandom
+     && p->head!=GAL_BLANK_SIZE_T
+     && p->tail!=GAL_BLANK_SIZE_T)
+    error(EXIT_FAILURE, 0, "only one of the following options can be "
+          "called in one run: '--head', '--tail', '--rowlimit' and "
+          "'--rowrandom'");
 
   /* If '--colmetadata' is given, make sure none of the given options have
      more than three values. */
@@ -298,7 +304,7 @@ ui_check_options_and_arguments(struct tableparams *p)
      a HDU is also given. */
   if(p->filename)
     {
-      if( gal_fits_name_is_fits(p->filename) && p->cp.hdu==NULL )
+      if( gal_fits_file_recognized(p->filename) && p->cp.hdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified. When the input is a FITS "
               "file, a HDU must also be specified, you can use the '--hdu' "
               "('-h') option and give it the HDU number (starting from "
@@ -461,8 +467,8 @@ ui_outcols_add_new_to_end(struct column_pack **list)
 
   /* Initialize its elements. */
   node->next=NULL;
+  node->arith=NULL;
   node->numsimple=0;
-  node->tokens=NULL;
   node->start=GAL_BLANK_SIZE_T;
 
   /* If the list already has elements, go to the last node in the list and
@@ -489,7 +495,7 @@ ui_outcols_free(struct column_pack *list)
   struct column_pack *tmp;
   while(list!=NULL)
     {
-      arithmetic_token_free(list->tokens);
+      arithmetic_token_free(list->arith);
       tmp=list->next;
       free(list);
       list=tmp;
@@ -620,7 +626,7 @@ ui_columns_prepare(struct tableparams *p)
               /* Add a new column pack, then read all the tokens (while
                  specifying which columns it needs). */
               node=ui_outcols_add_new_to_end(&p->outcols);
-              arithmetic_init(p, &node->tokens, &toread, &totcalled,
+              arithmetic_init(p, &node->arith, &toread, &totcalled,
                               strarr[i]+ARITHMETIC_CALL_LENGTH);
               free(strarr[i]);
             }
@@ -636,7 +642,7 @@ ui_columns_prepare(struct tableparams *p)
                   /* If the previous column package was an arithmetic
                      operation, allocate a new node. */
                   last=ui_outcols_last(p->outcols);
-                  if(last->tokens)
+                  if(last->arith)
                     {
                       node=ui_outcols_add_new_to_end(&p->outcols);
                       node->start=totcalled;
@@ -671,28 +677,29 @@ ui_columns_prepare(struct tableparams *p)
       struct column_pack *tmp;
       struct arithmetic_token *atmp;
       for(tmp=p->outcols;tmp!=NULL;tmp=tmp->next)
-        {
-          if(tmp->tokens)
-            for(atmp=tmp->tokens;atmp!=NULL;atmp=atmp->next)
+        if(tmp->arith)
+          {
+            printf("Arithmetic: \n");
+            for(atmp=tmp->arith;atmp!=NULL;atmp=atmp->next)
               {
-                printf("Arithmetic: ");
-                if(atmp->constant) printf("Constant number\n");
-                else if(atmp->index) printf("Called column: %zu\n",
-                                            atmp->index);
-                else if(atmp->operator!=ARITHMETIC_TABLE_OP_INVALID)
-                  printf("Operator: %d\n", atmp->operator);
-                else
-                  error(EXIT_FAILURE, 0, "%s: UNKNOWN SITUATION!",
-                        __func__);
+                if(atmp->operator!=GAL_ARITHMETIC_OP_INVALID)
+                  {
+                    printf("\tOperator: %d\n", atmp->operator);
+                    if(atmp->name_def)
+                      printf("\t\t(Name definition: %s)\n", atmp->name_def);
+                  }
+                else if(atmp->constant) printf("\tConstant number\n");
+                else if(atmp->name_use) printf("\tName usage: %s\n",
+                                               atmp->name_use);
+                else printf("\tCalled column: %zu\n", atmp->index);
               }
-          else
-            printf("Simple: start: %zu, num: %zu\n", tmp->start,
-                   tmp->numsimple);
-        }
+          }
+        else
+          printf("Simple: start: %zu, num: %zu\n", tmp->start,
+                 tmp->numsimple);
     }
   */
 
-
   /* Delete the old list, then reverse the 'toread' list, and put it into
      'p->columns'. */
   gal_list_str_free(p->columns, 1);
@@ -1042,7 +1049,8 @@ ui_check_select_sort_after(struct tableparams *p, size_t nselect,
 static void
 ui_preparations(struct tableparams *p)
 {
-  size_t *colmatch;
+  double *darr;
+  size_t i, *colmatch;
   gal_list_str_t *lines;
   size_t nselect=0, origoutncols=0;
   size_t sortindout=GAL_BLANK_SIZE_T;
@@ -1132,6 +1140,51 @@ ui_preparations(struct tableparams *p)
               ? p->table->size
               : p->tail );
 
+  /* If rows are given, do some sanity checks and make sure that they are
+     within the table's limits. */
+  if(p->rowlimit)
+    {
+      /* There should only be two values. */
+      if(p->rowlimit->size!=2)
+        error(EXIT_FAILURE, 0, "only two should be given to "
+              "'--rowlimit' (the top and bottom row numbers specifying "
+              "your desired range)");
+
+      /* Do individual checks. */
+      darr=p->rowlimit->array;
+      for(i=0;i<p->rowlimit->size;++i)
+        {
+          /* Make sure it isn't 0 or negative. */
+          if( darr[i]<=0 )
+            error(EXIT_FAILURE, 0, "%g (value given to '--rowlimit') "
+                  "is smaller than, or equal to, zero! This option's "
+                  "values are row-counters (starting from 1), so they "
+                  "must be positive integers", darr[i]);
+
+          /* Make sure its an integer. */
+          if( darr[i] != (size_t)(darr[i]) )
+            error(EXIT_FAILURE, 0, "%g (value given to '--rowlimit') is "
+                  "not an integer! This option's values are row-counters "
+                  "so they must be integers.", darr[i]);
+
+          /* Subtract 1 from the value, so it counts from 0. */
+          --darr[i];
+        }
+
+      /* Make sure that the first value is smaller than the second. */
+      if( darr[0] > darr[1] )
+        error(EXIT_FAILURE, 0, "the first value to '--rowlimit' (%g) is "
+              "larger than the second (%g). This option's values defines "
+              "a row-counter interval, assuming the first value is the top "
+              "of the desired interval (smaller row counter) and the second "
+              "value is the bottom of the desired interval (larger row "
+              "counter)", darr[0], darr[1]);
+    }
+
+  /* If random rows are desired, we need to define a GSL random number
+     generator structure. */
+  if(p->rowrandom)
+    p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
 
   /* Clean up. */
   free(colmatch);
@@ -1211,6 +1264,19 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct tableparams *p)
 
   /* Read/allocate all the necessary starting arrays. */
   ui_preparations(p);
+
+  /* Let the user know basic information if necessary (for example when a
+     random number generator has been used). */
+  if(p->rng && !p->cp.quiet)
+    {
+      /* Write the information. */
+      printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
+             ctime(&p->rawtime));
+      printf("Parameters used for '--randomrows':\n");
+      printf("  - Random number generator name: %s\n", p->rng_name);
+      printf("  - Random number generator seed: %lu\n", p->rng_seed);
+      printf("(use '--quiet' to suppress this starting message)\n");
+    }
 }
 
 
@@ -1245,4 +1311,8 @@ ui_free_report(struct tableparams *p)
   gal_list_data_free(p->table);
   gal_list_data_free(p->colmetadata);
   if(p->colarray) free(p->colarray);
+
+  /* If a random number generator was allocated, free it. */
+  if(p->rng) gsl_rng_free(p->rng);
+
 }
