@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2019, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -55,7 +55,7 @@ binary_erode_dilate_2d_4con(gal_data_t *input, int dilate0_erode1)
   /* Do a sanity check: */
   if(dilate0_erode1!=1 && dilate0_erode1!=0)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can "
-          "fix this problem. The value to `dilate0_erode1' is %u while it "
+          "fix this problem. The value to 'dilate0_erode1' is %u while it "
           "should be 0 or 1", __func__, PACKAGE_BUGREPORT, dilate0_erode1);
 
   /* Set the foreground and background values. */
@@ -259,15 +259,15 @@ binary_erode_dilate_general(gal_data_t *input, unsigned char dilate0_erode1,
 
 
 
-/* Erode a binary dataset any number of times. If `inplace' is given a
-   value of `1', then do the erosion within the already allocated space,
+/* Erode a binary dataset any number of times. If 'inplace' is given a
+   value of '1', then do the erosion within the already allocated space,
    otherwise, allocate a new array and save the result into that.
 
    This function will only work on the elements with a value of 1 or 0. It
    will leave all the rest unchanged. Also note that it only works on
-   `uint8_t' type datasets. So if the input doesn't have that type, it is
+   'uint8_t' type datasets. So if the input doesn't have that type, it is
    going to copy it this type and return the newlyallocated dataset. So
-   when the input's type isn't `uint8_t', `inplace' is irrelevant. */
+   when the input's type isn't 'uint8_t', 'inplace' is irrelevant. */
 static gal_data_t *
 binary_erode_dilate(gal_data_t *input, size_t num, int connectivity,
                     int inplace, int d0e1)
@@ -279,7 +279,7 @@ binary_erode_dilate(gal_data_t *input, size_t num, int connectivity,
   /* Currently this only works on blocks. */
   if(input->block)
     error(EXIT_FAILURE, 0, "%s: currently only works on a fully "
-          "allocated block of memory, but the input is a tile (its `block' "
+          "allocated block of memory, but the input is a tile (its 'block' "
           "element is not NULL)", __func__);
 
   /* Set the dataset to work on. */
@@ -352,8 +352,8 @@ gal_binary_open(gal_data_t *input, size_t num, int connectivity,
   /* First do the necessary number of erosions. */
   out=gal_binary_erode(input, num, connectivity, inplace);
 
-  /* If `inplace' was called, then `out' is the same as `input', if it
-     wasn't, then `out' is a newly allocated array. In any case, we should
+  /* If 'inplace' was called, then 'out' is the same as 'input', if it
+     wasn't, then 'out' is a newly allocated array. In any case, we should
      dilate in the same allocated space. */
   gal_binary_dilate(input, num, connectivity, 1);
 
@@ -397,7 +397,7 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
 
   /* Two small sanity checks. */
   if(binary->type!=GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "%s: the input data set type must be `uint8'",
+    error(EXIT_FAILURE, 0, "%s: the input data set type must be 'uint8'",
           __func__);
   if(binary->block)
     error(EXIT_FAILURE, 0, "%s: currently, the input data structure to "
@@ -412,13 +412,13 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
 
       /* Make sure the given dataset has the same size as the input. */
       if( gal_dimension_is_different(binary, lab) )
-        error(EXIT_FAILURE, 0, "%s: the `binary' and `out' datasets must "
+        error(EXIT_FAILURE, 0, "%s: the 'binary' and 'out' datasets must "
               "have the same size", __func__);
 
-      /* Make sure it has a `int32' type. */
+      /* Make sure it has a 'int32' type. */
       if( lab->type!=GAL_TYPE_INT32 )
-        error(EXIT_FAILURE, 0, "%s: the `out' dataset must have `int32' type"
-              "but the array you have given is `%s' type", __func__,
+        error(EXIT_FAILURE, 0, "%s: the 'out' dataset must have 'int32' type"
+              "but the array you have given is '%s' type", __func__,
               gal_type_name(lab->type, 1));
 
       /* Reset all its values to zero. */
@@ -491,7 +491,7 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
 
 
 
-/* Put the indexs of connected labels in a list of `gal_data_t's, each with
+/* Put the indexs of connected labels in a list of 'gal_data_t's, each with
    a one-dimensional array that has the indexs of that connected
    component.*/
 #define BINARY_CONINDEX_VAL 2
@@ -506,7 +506,7 @@ gal_binary_connected_indexs(gal_data_t *binary, int connectivity)
 
   /* Small sanity checks. */
   if(binary->type!=GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "%s: the input data set type must be `uint8'",
+    error(EXIT_FAILURE, 0, "%s: the input data set type must be 'uint8'",
           __func__);
   if(binary->block)
     error(EXIT_FAILURE, 0, "%s: currently, the input data structure to "
@@ -516,7 +516,7 @@ gal_binary_connected_indexs(gal_data_t *binary, int connectivity)
   b=binary->array;
   for(i=0;i<binary->size;++i)
     /* A pixel that has already been recorded is given a value of
-       `BINARY_CONINDEX_VAL'. */
+       'BINARY_CONINDEX_VAL'. */
     if( b[i]==1 )
       {
         /* Add this pixel to the queue of pixels to work with. */
@@ -569,7 +569,7 @@ gal_binary_connected_indexs(gal_data_t *binary, int connectivity)
   }
   */
 
-  /* Set all the `2' values back to `1'. */
+  /* Set all the '2' values back to '1'. */
   bf=(b=binary->array)+binary->size;
   do if(*b==BINARY_CONINDEX_VAL) *b=1; while(++b<bf);
 
@@ -597,8 +597,8 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
 
   /* Some small sanity checks. */
   if(adjacency->type != GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "%s: input must have type `uint8'. However, the "
-          "input dataset has type of `%s'", __func__,
+    error(EXIT_FAILURE, 0, "%s: input must have type 'uint8'. However, the "
+          "input dataset has type of '%s'", __func__,
           gal_type_name(adjacency->type, 1));
 
   if(adjacency->ndim != 2)
@@ -653,7 +653,6 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
         ++curlab;
       }
 
-
   /* For a check.
   printf("=== Old labels --> new labels ===\n");
   for(i=1;i<num;++i) printf("%zu: %u\n", i, newlabs[i]);
@@ -668,6 +667,84 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
 
 
 
+/* List-based adjacency matrix: when the number of points to find adjacent
+   elements is large, the array-based adjacency solution of
+   'gal_binary_connected_adjacency_matrix' will consume far too much memory
+   (since it is a square). In those cases, we need to work based on
+   lists.
+
+   The input is an array of 'size_t' list pointers. Each one points to the
+   labels that are touching it.
+
+                     indexs        b       d
+                                   ^       ^
+                     indexs        a   b   c   a
+                                   ^   ^   ^   ^
+                     listarr:    | * | * | * | * |
+*/
+gal_data_t *
+gal_binary_connected_adjacency_list(gal_list_sizet_t **listarr,
+                                    size_t number, size_t minmapsize,
+                                    int quietmmap, size_t *numconnected)
+{
+  size_t i, p;
+  gal_list_sizet_t *tmp;
+  gal_data_t *newlabs_d;
+  gal_list_sizet_t *Q=NULL;
+  int32_t *newlabs, curlab=1;
+
+  /* Allocate (and clear) the output datastructure. */
+  newlabs_d=gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &number, NULL, 1,
+                           minmapsize, quietmmap, NULL, NULL, NULL);
+  newlabs=newlabs_d->array;
+
+  /* Go over the input matrix and apply the same principle as we used to
+     identify connected components in an image: through a queue, find those
+     elements that are connected. */
+  for(i=1;i<number;++i)
+    if(newlabs[i]==0)
+      {
+        /* Add this old label to the list that must be corrected. */
+        gal_list_sizet_add(&Q, i);
+
+        /* Continue while the list has elements. */
+        while(Q!=NULL)
+          {
+            /* Pop the top old-label from the list. */
+            p=gal_list_sizet_pop(&Q);
+
+            /* If it has already been labeled then ignore it. */
+            if( newlabs[p]!=curlab )
+              {
+                /* Give it the new label. */
+                newlabs[p]=curlab;
+
+                /* Go over the adjacency list for this touching object and
+                   see if there are any not-yet-labeled objects that are
+                   touching it. */
+                for(tmp=listarr[p]; tmp!=NULL; tmp=tmp->next)
+                  if( newlabs[tmp->v]==0 )
+                    gal_list_sizet_add(&Q, tmp->v);
+              }
+          }
+
+        /* Increment the current label. */
+        ++curlab;
+      }
+
+  /* For a check.
+  printf("=== Old labels --> new labels ===\n");
+  for(i=1;i<number;++i) printf("%zu: %u\n", i, newlabs[i]);
+  */
+
+  /* Return the output. */
+  *numconnected = curlab-1;
+  return newlabs_d;
+}
+
+
+
+
 
 
 
@@ -766,8 +843,8 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
 
   /* A small sanity check. */
   if( input->type != GAL_TYPE_UINT8 )
-    error(EXIT_FAILURE, 0, "%s: input must have `uint8' type, but its "
-          "input dataset has `%s' type", __func__,
+    error(EXIT_FAILURE, 0, "%s: input must have 'uint8' type, but its "
+          "input dataset has '%s' type", __func__,
           gal_type_name(input->type, 1));
 
 
@@ -785,13 +862,13 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
      we should invert the respective pixel. To do it, we'll use the tile
      that was defined before, just change its block and array.*/
   tile->array=gal_tile_block_relative_to_other(tile, holelabs);
-  tile->block=holelabs; /* has to be after correcting `tile->array'. */
+  tile->block=holelabs; /* has to be after correcting 'tile->array'. */
 
 
-  /* The type of the tile is already known (it is `int32_t') and we have no
-     output/other, so we'll just put `int' as a place-holder. In this way
+  /* The type of the tile is already known (it is 'int32_t') and we have no
+     output/other, so we'll just put 'int' as a place-holder. In this way
      we can avoid the switch statement of GAL_TILE_PARSE_OPERATE, and
-     directly use the workhorse macro `GAL_TILE_PO_OISET'. */
+     directly use the workhorse macro 'GAL_TILE_PO_OISET'. */
   lab=(holelabs)->array;
   GAL_TILE_PO_OISET(int32_t, int, tile, NULL, 0, 0, {
       *lab++ = ( *i
@@ -812,7 +889,7 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
      from the start, effectively removing the paddings. Therefore, ee will
      just correct the sizes and we won't bother actually re-allocating the
      array size in memory. According to the GNU C library's description
-     after `realloc': "In several allocation implementations, making a
+     after 'realloc': "In several allocation implementations, making a
      block smaller sometimes necessitates copying it, so it can fail if no
      other space is available.". The extra padding is only 2 pixels wide,
      thus, the extra space is negligible compared to the actual array. So
@@ -850,7 +927,7 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
 
    3. Since we had a 2 pixel padding on the edges of the image, we
       know for sure that all labeled regions with a label of 1 are
-      actually connected `holes' in the input image.
+      actually connected 'holes' in the input image.
 
       Any pixel with a label larger than 1, is therefore a bounded
       hole that is not 8-connected to the rest of the holes.  */
@@ -864,8 +941,8 @@ gal_binary_holes_fill(gal_data_t *input, int connectivity, size_t maxsize)
 
   /* Small sanity checks. */
   if( input->type != GAL_TYPE_UINT8 )
-    error(EXIT_FAILURE, 0, "%s: input must have `uint8' type, but its "
-          "input dataset has `%s' type", __func__,
+    error(EXIT_FAILURE, 0, "%s: input must have 'uint8' type, but its "
+          "input dataset has '%s' type", __func__,
           gal_type_name(input->type, 1));
   if(connectivity<1 || connectivity>input->ndim)
     error(EXIT_FAILURE, 0, "%s: connectivity value %d is not acceptable. "
@@ -886,7 +963,7 @@ gal_binary_holes_fill(gal_data_t *input, int connectivity, size_t maxsize)
      that was defined before, just change its block and array.*/
   in=input->array;
   tile->array=gal_tile_block_relative_to_other(tile, holelabs);
-  tile->block=holelabs; /* has to be after correcting `tile->array'. */
+  tile->block=holelabs; /* has to be after correcting 'tile->array'. */
 
   /* If the user wants to only fill holes to a certain size, then remove
      those with a larger size. */
@@ -908,10 +985,10 @@ gal_binary_holes_fill(gal_data_t *input, int connectivity, size_t maxsize)
       free(sizes);
     }
 
-  /* The type of the tile is already known (it is `int32_t') and we have no
-     output, so we'll just put `int' as a place-holder. In this way we can
+  /* The type of the tile is already known (it is 'int32_t') and we have no
+     output, so we'll just put 'int' as a place-holder. In this way we can
      avoid the switch statement of GAL_TILE_PARSE_OPERATE, and directly use
-     the workhorse macro `GAL_TILE_PO_OISET'. */
+     the workhorse macro 'GAL_TILE_PO_OISET'. */
   GAL_TILE_PO_OISET(int32_t, int, tile, NULL, 0, 0, {
       *in = *i>1 && *i!=GAL_BLANK_INT32 ? 1 : *in;
       ++in;
