@@ -62,16 +62,43 @@ gal_units_extract_decimal(char *convert, const char *delimiter,
                           double *args, size_t n);
 
 double
-gal_units_ra_to_degree (char *convert);
+gal_units_ra_to_degree(char *convert);
 
 double
-gal_units_dec_to_degree (char *convert);
+gal_units_dec_to_degree(char *convert);
 
 char *
-gal_units_degree_to_ra (double decimal, int usecolon);
+gal_units_degree_to_ra(double decimal, int usecolon);
 
 char *
-gal_units_degree_to_dec (double decimal, int usecolon);
+gal_units_degree_to_dec(double decimal, int usecolon);
+
+double
+gal_units_counts_to_mag(double counts, double zeropoint);
+
+double
+gal_units_mag_to_counts(double mag, double zeropoint);
+
+double
+gal_units_counts_to_jy(double counts, double zeropoint_ab);
+
+double
+gal_units_au_to_pc(double au);
+
+double
+gal_units_pc_to_au(double pc);
+
+double
+gal_units_ly_to_pc(double ly);
+
+double
+gal_units_pc_to_ly(double pc);
+
+double
+gal_units_ly_to_au(double ly);
+
+double
+gal_units_au_to_ly(double au);
 
 __END_C_DECLS    /* From C++ preparations */
 
