@@ -54,4 +54,4 @@ if [ ! -f $cat      ]; then echo "$cat does not exist.";   exit 77; fi
 # string. Such programs will execute the command if present and help in
 # debugging when the developer doesn't have access to the user's system.
 $check_with_program $execname $cat --ccol=RA --ccol=Dec --mode=wcs   \
-                              --mergedsize=100,100
+                    --mergedsize=100,100 --output=radeccat.fits
